package com.emato.cuspay.wx.declare;

import com.emato.cuspay.base.merch.MerchCusService;
import com.emato.cuspay.base.merch.MerchNotiService;
import com.emato.cuspay.common.contant.MerchNoticeDict;
import com.emato.cuspay.dto.merch.MerchCusCfg;
import com.emato.cuspay.wx.common.MerchNotiBuilder;
import com.emato.cuspay.wx.common.WxContants;
import com.emato.cuspay.wx.common.WxDict;
import com.emato.cuspay.common.contant.TablePrimaryKeyPrefix;
import com.emato.cuspay.common.core.db.IdWorkerAide;
import com.emato.cuspay.dao.mapper.merch.MerchNotiMapper;
import com.emato.cuspay.wx.dto.WxResponseMsg;
import com.emato.cuspay.dto.merch.MerchNoti;
import com.emato.cuspay.dto.merch.MerchPayCfg;
import com.emato.cuspay.wx.dao.entity.WxCbPayDoc;
import com.emato.cuspay.wx.dao.entity.WxPayError;
import com.emato.cuspay.util.OkHttpUtils;
import com.emato.cuspay.util.XmlUtils;
import com.emato.cuspay.wx.service.WxCbPayDocService;
import com.emato.cuspay.wx.service.WxPayErrorService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.xml.bind.JAXB;
import java.io.IOException;
import java.io.StringReader;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;

/**
 * 微信海关申报
 * @author zx
 * @version 1.0
 * 2018-05-18 11:04
 */

@Component
public class WxCusDeclare extends AbstractCusDeclare implements CusDeclare{

    private static final Logger logger = LoggerFactory.getLogger(WxCusDeclare.class);


    @Value("${db.wx.declare.limit}")
    private Integer limit;

    @Value("${db.merch.notice.count}")
    private Integer count;

    @Value("${wx.payment.declare.url}")
    private String declareURL;


    @Autowired
    private WxCbPayDocService wxCbPayDocService;

    @Autowired
    private WxPayErrorService wxPayErrorService;

    @Autowired
    private MerchNotiService merchNotiService;

    @Autowired
    private MerchCusService merchCusService;

    /**
     * 海关报关接口
     */
    public void declare() {
        limit = (limit != null && limit > 0) ? limit : 20;
        count = (count != null && count > 0) ? count : 10;

        List<WxCbPayDoc> wxCbPayDocs = wxCbPayDocService.selectBeDeclaredData(limit);

        if (wxCbPayDocs == null || wxCbPayDocs.isEmpty()) {
            logger.info("没有待申报微信推海关的支付数据");
            return;
        }

        //更新状态申报中
        wxCbPayDocs.forEach(wxCbPayDoc -> {
           wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_01.getItem());
        });

        try {
            int reslut = wxCbPayDocService.updateBatch(wxCbPayDocs);
            if (reslut < 0) {
                throw new Exception("批量更新失败影响行数为0");
            }
        } catch(Exception e)  {
            logger.error("微信报关数据批量更失败data："+wxCbPayDocs, e);
            return;
        }


        //记录异常数据
        List<WxPayError> errors = Lists.newArrayList();
        //待通知的用户
        List<MerchNoti> merchNotis = Lists.newArrayList();

        //修改数支付单状态为申报中

        wxCbPayDocs.forEach(wxCbPayDoc -> {

            //获取商户信息 从缓存中去  缓存不存在 从数据库重取
            MerchPayCfg merchPayCfg = getMerchPayCfgCache(wxCbPayDoc.getMerchSn());

            MerchNotiBuilder builder = new MerchNotiBuilder();

            //设置支付信息
            builder.wxCbPay(wxCbPayDoc);
            //设置通知次数
            builder.notiCount(count);

            if (merchPayCfg == null) {
                logger.error("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付配置信息不存在");
                wxCbPayDoc.setReturnCode(WxContants.ERROR_NO_INFOMATION);
                wxCbPayDoc.setReturnMsg("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付配置信息不存在");
                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                MerchNoti merchNoti = builder.code(WxContants.ERROR_NO_INFOMATION)
                                .msg("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付配置信息不存在")
                                .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                                .build();
                merchNotis.add(merchNoti);
                return;
            }

            if (merchPayCfg.getMerchWxApiKey() == null) {
                logger.error("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付信息api密钥为空");
                wxCbPayDoc.setReturnCode(WxContants.ERROR_NO_INFOMATION);
                wxCbPayDoc.setReturnMsg("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付配置信息不存在");
                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                MerchNoti merchNoti = builder.code(WxContants.ERROR_NO_API_KEY)
                        .msg("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付信息api密钥为空")
                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                        .build();
                merchNotis.add(merchNoti);
                return;
            }
            //设置商户配置信息
            builder.merchPay(merchPayCfg);

            //组装xml 格式数据
            String xml = XmlUtils.map2Xml(assemblyDeclareXml(wxCbPayDoc, merchPayCfg.getMerchWxApiKey()));
            logger.info("微信支付报关请求数xml据为:"+xml);

            //通过http post请求 发送xml数据
            Request request = OkHttpUtils.buildRequest(declareURL,
                        RequestBody.create(MediaType.parse("application/xml; charset=utf-8"), xml));
            String result = null;
            try {
                result = OkHttpUtils.post(request);
                StringReader reader = new StringReader(result);
                WxResponseMsg wxResponseMsgDto = JAXB.unmarshal(reader, WxResponseMsg.class);
                logger.info("result:"+result + "wx response message pojo:"+wxResponseMsgDto);
                if (wxResponseMsgDto == null) {
                    throw new IOException("http请求["+declareURL+"]响应结果为空" );
                }
                ///返回状态码为失败
                if (WxContants.RETURN_CODE_FAIL.equals(wxResponseMsgDto.getReturnCode()) ||
                        WxContants.FAIL.equals(wxResponseMsgDto.getReturnCode())) {

                    wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                    wxCbPayDoc.setResultCode(wxResponseMsgDto.getReturnCode());
                    wxCbPayDoc.setReturnMsg(wxResponseMsgDto.getReturnMsg());
                    wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                    //通知商户
                    MerchNoti merchNoti = builder.code(WxContants.ERROR_FAIL)
                                    .msg(wxResponseMsgDto.getReturnMsg())
                                    .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                                    .build();
                    merchNotis.add(merchNoti);
                }

                //返回这状态码为成功 业务结果成功
                if (WxContants.SUCCESS.equals(wxResponseMsgDto.getReturnCode())
                            && WxContants.SUCCESS.equals(wxResponseMsgDto.getResultCode())) {
                    wxCbPayDoc.setReturnCode(wxResponseMsgDto.getReturnCode());
                    wxCbPayDoc.setReturnMsg(wxResponseMsgDto.getReturnMsg());
                    wxCbPayDoc.setResultCode(wxResponseMsgDto.getResultCode());

                    //订购人与支付人校验结果
                    if (wxResponseMsgDto.getCertCheckResult() != null) {
                        if ("UNCHECKED".equals(wxResponseMsgDto.getCertCheckResult())) { //未知
                            builder.certCheckResult(MerchNoticeDict.BuyerPayerCheckStatus.i_0.getItem());
                        }
                        if ("SAME".equals(wxResponseMsgDto.getCertCheckResult())) {//一致
                            builder.certCheckResult(MerchNoticeDict.BuyerPayerCheckStatus.i_1.getItem());
                        }
                        if ("DIFFERENT".equals(wxResponseMsgDto.getCertCheckResult())) {//不一致
                            builder.certCheckResult(MerchNoticeDict.BuyerPayerCheckStatus.i_2.getItem());
                        }
                    }


                    //返回状态未申报
                    if (WxDict.ResponseMsgState.UNDECLARED.getItem().equals(wxResponseMsgDto.getState())) {
                        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_01.getItem());
                        wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                        MerchNoti merchNoti = builder.code(WxContants.ERROR_FAIL)
                                        .msg("未申报")
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                                        .build();
                        merchNotis.add(merchNoti);
                    }

                    //申报中
                    if (WxDict.ResponseMsgState.PROCESSING.getItem().equals(wxResponseMsgDto.getState())) {
                        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_01.getItem());
                        wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                        MerchNoti merchNoti = builder.code(wxResponseMsgDto.getErrCode())
                                        .msg(wxResponseMsgDto.getErrCodeDes())
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_11.getItem())
                                        .build();
                        merchNotis.add(merchNoti);
                    }

                    //申报已提交
                    if (WxDict.ResponseMsgState.SUBMITTED.getItem().equals(wxResponseMsgDto.getState())) {
                        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_10.getItem());
                        wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                        MerchNoti merchNoti = builder.code(wxResponseMsgDto.getErrCode())
                                .msg(wxResponseMsgDto.getErrCodeDes())
                                .cusDeclStatus(WxDict.MerchNoticeStatus.i_11.getItem())
                                .build();
                        merchNotis.add(merchNoti);

                    }
                    //申报成功
                    if (WxDict.ResponseMsgState.SUCCESS.getItem().equals(wxResponseMsgDto.getState())) {
                        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_11.getItem());
                        wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                        MerchNoti merchNoti = builder.code(wxResponseMsgDto.getErrCode())
                                .msg(wxResponseMsgDto.getErrCodeDes())
                                .cusDeclStatus(WxDict.MerchNoticeStatus.i_13.getItem())
                                .build();
                        merchNotis.add(merchNoti);
                    }
                    //申报失败
                    if (WxDict.ResponseMsgState.FAIL.getItem().equals(wxResponseMsgDto.getState())) {
                        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                        wxCbPayDocService.updateWxCbPay(wxCbPayDoc);


                        MerchNoti merchNoti = builder.code(WxContants.ERROR_FAIL)
                                    .msg("海关申报失败")
                                    .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                                .build();
                        merchNotis.add(merchNoti);
                    }
                    //海关接口异常
                    if (WxDict.ResponseMsgState.EXCEPT.getItem().equals(wxResponseMsgDto.getState())) {
                        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                        wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                        MerchNoti merchNoti = builder.code(WxContants.ERROR_EXCEPT)
                                    .msg("海关接口异常")
                                    .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                                    .build();
                        merchNotis.add(merchNoti);
                    }


                }

                //返回状态码为成功 业务结果失败
                if (WxContants.SUCCESS.equals(wxResponseMsgDto.getReturnCode())
                            && WxContants.FAIL.equals(wxResponseMsgDto.getResultCode())) {
                    wxCbPayDoc.setResultCode(wxResponseMsgDto.getResultCode());
                    wxCbPayDoc.setErrCode(wxResponseMsgDto.getErrCode());
                    wxCbPayDoc.setErrCodeDes(wxResponseMsgDto.getErrCodeDes());
                    wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_12.getItem());
                    wxCbPayDocService.updateWxCbPay(wxCbPayDoc);

                    MerchNoti merchNoti = builder.code(wxResponseMsgDto.getErrCode())
                                .msg(wxResponseMsgDto.getErrCodeDes())
                                .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                                .build();
                    merchNotis.add(merchNoti);
                }

            } catch (IOException e) {
                logger.error("数据请求异常的支付单请求数据为xml:"+xml, e);
                WxPayError wxPayError = createWxPayError(wxCbPayDoc);
                errors.add(wxPayError);
            }
        });

        //持久化商户通知数据
        try {
            int result = merchNotiService.insertBatch(merchNotis);
            if (result < 0) {
                logger.error("持久化" + result + "条商户通知数据失败data:" + merchNotis);
                throw new Exception("持久化" + result + "条商户通知数据失败data:" + merchNotis);
            }
        } catch (Exception e) {
            logger.error("持久化商户通知数据异常", e);
            return;
        }
        //存储异常记录
        try {
            if (errors != null && !errors.isEmpty()) {
                int errResult = wxPayErrorService.insertWxPayErrorBatch(errors);
                if (errResult < 0) {
                    logger.error("持久化支付申报异常信息数据失败data:" + errors);
                    throw new Exception("持久化支付申报异常信息数据失败data:" + errors);
                }
            }
        } catch (Exception e) {
            logger.error("持久化支付申报异常信息数据失败data:" + errors, e);
            return;
        }
    }

    /**
     * 记录异常的支付单证信息
     * @param wxCbPayDoc
     * @return
     */
    private WxPayError createWxPayError(WxCbPayDoc wxCbPayDoc) {
        WxPayError wxPayError = new WxPayError();
        wxPayError.setErrorSn(TablePrimaryKeyPrefix.wx_pay_error_type + IdWorkerAide.nextId());
        wxPayError.setWxPaySn(wxCbPayDoc.getTransactionId());
        wxPayError.setMerchSn(wxCbPayDoc.getMerchSn());
        wxPayError.setMerchName(wxCbPayDoc.getMerchName());
        wxPayError.setAppid(wxCbPayDoc.getAppid());
        wxPayError.setMchId(wxCbPayDoc.getMchId());
        wxPayError.setOutTradeNo(wxCbPayDoc.getOutTradeNo());
        wxPayError.setTransactionId(wxCbPayDoc.getTransactionId());
        wxPayError.setSubOrderNo(wxCbPayDoc.getSubOrderNo());
        wxPayError.setCustoms(wxCbPayDoc.getCustoms());
        wxPayError.setErrCode(wxCbPayDoc.getErrCode());
        wxPayError.setErrMsg(wxCbPayDoc.getReturnMsg());
        //TODO:
        wxPayError.getCreateTime();
        return  wxPayError;
    }



    /**
     * 组装报关所需要的报关数据
     * @param wxCbPayDoc 微信支付数据
     * @param key 微信Api密钥
     * @return
     */
    protected Map<String, String> assemblyDeclareXml(WxCbPayDoc wxCbPayDoc, String key) {


        //1.选择报关数据,排序
        SortedMap<String, String> sorted = Maps.newTreeMap();
        sorted.put("appid", wxCbPayDoc.getAppid());
        sorted.put("mch_id", wxCbPayDoc.getMchId());
        sorted.put("out_trade_no", wxCbPayDoc.getOutTradeNo());
        sorted.put("transaction_id", wxCbPayDoc.getTransactionId());
        sorted.put("customs", wxCbPayDoc.getCustoms());
        sorted.put("mch_customs_no", wxCbPayDoc.getMchCustomsNo());
        if (wxCbPayDoc.getDuty() != null) {
            sorted.put("duty", String.valueOf(wxCbPayDoc.getDuty()));
        }
        if (wxCbPayDoc.getActionType() != null && !wxCbPayDoc.getActionType().isEmpty()) {
            sorted.put("action_type", wxCbPayDoc.getActionType());
        }
        if (wxCbPayDoc.getSubOrderNo() != null && !wxCbPayDoc.getSubOrderNo().isEmpty()) {
            sorted.put("sub_order_no", wxCbPayDoc.getActionType());
        }
        if (wxCbPayDoc.getFeeType() != null && !wxCbPayDoc.getFeeType().isEmpty()) {
            sorted.put("fee_type", wxCbPayDoc.getFeeType());
        }
        if (wxCbPayDoc.getOrderFee() != null) {
            sorted.put("order_fee", String.valueOf(wxCbPayDoc.getOrderFee()));
        }
        if (wxCbPayDoc.getTransportFee() != null) {
            sorted.put("transport_fee", String.valueOf(wxCbPayDoc.getTransportFee()));
        }
        if (wxCbPayDoc.getProductFee() != null) {
            sorted.put("product_fee", String.valueOf(wxCbPayDoc.getProductFee()));
        }
        if (wxCbPayDoc.getCertType() != null && !wxCbPayDoc.getCertType().isEmpty()) {
            sorted.put("cert_type", wxCbPayDoc.getCertType());
        }
        if (wxCbPayDoc.getCertId() != null && !wxCbPayDoc.getCertId().isEmpty()) {
            sorted.put("cert_id", wxCbPayDoc.getCertId());
        }
        if (wxCbPayDoc.getName() != null && !wxCbPayDoc.getName().isEmpty()) {
            sorted.put("name", wxCbPayDoc.getName());
        }

        //2.生成签名
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String,String> entry : sorted.entrySet()) {
            sb.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
        }
        sb.append("key").append("=").append(key);
        //签名MD5 加密
        String sign = DigestUtils.md5Hex(sb.toString()).toUpperCase();

        sorted.put("sign", sign);
        return sorted;
    }

    public static void main(String[] args) {
        WxCbPayDoc wxCbPayDoc = new WxCbPayDoc();
        wxCbPayDoc.setAppid("wx24013b34e349db3c");
        wxCbPayDoc.setMchId("1501125641");
        wxCbPayDoc.setOutTradeNo("669918012250");
        wxCbPayDoc.setTransactionId("4200000120201805021020091366");
        wxCbPayDoc.setCustoms("SHENZHEN");
        wxCbPayDoc.setMchCustomsNo("4403160Z3Y");//中网科技(深圳)有限公司


        SortedMap<String, String> sorted = Maps.newTreeMap();
        sorted.put("appid", wxCbPayDoc.getAppid());
        sorted.put("mch_id", wxCbPayDoc.getMchId());
        sorted.put("out_trade_no", wxCbPayDoc.getOutTradeNo());
        sorted.put("transaction_id", wxCbPayDoc.getTransactionId());
        sorted.put("customs", wxCbPayDoc.getCustoms());
        sorted.put("mch_customs_no", wxCbPayDoc.getMchCustomsNo());

        //2.生成签名
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String,String> entry : sorted.entrySet()) {
            sb.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
        }
        sb.append("key").append("=").append("ZC4g5ryxTQelj3x556HwljSpSX4Enzsy");
        //签名MD5 加密
        String sign = DigestUtils.md5Hex(sb.toString()).toUpperCase();

        sorted.put("sign", sign);

        String xml = XmlUtils.map2Xml(sorted);

        System.out.println("xml:"+xml);

        //通过http post请求 发送xml数据
        Request request = OkHttpUtils.buildRequest("https://api.mch.weixin.qq.com/cgi-bin/mch/customs/customdeclareorder",
                RequestBody.create(MediaType.parse("application/xml; charset=utf-8"), xml));
        String result = null;
        try {
            result = OkHttpUtils.post(request);
        } catch (IOException e) {
            e.printStackTrace();
        }



        StringReader reader = new StringReader(result);

        WxResponseMsg wxResponseMsg = JAXB.unmarshal(reader, WxResponseMsg.class);

        System.out.println(wxResponseMsg);
    }
}
